﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace Chapter_17
{
    class Program
    {
        static void Main()
        {
            DemoListOfString();
            DemoArrayList();
            DemoListOfCustomer();
            DemoArrayListBoxingPenalty();
            DemoListAvoidingBoxingPenalty();
            DemoShapeList();
            DemoGenericSingleLinkedList();
            DemoGenericInterface();
            DemoGenericDelegate();
            DemoConstraints();

            Console.ReadKey();
        }

        private static void DemoListOfString()
        {
            var letters = new List<string>();

            letters.Add("Alpha");
            letters.Add("Beta");
            letters.Add("Charlie");

            foreach (var letter in letters)
            {
                Console.WriteLine(letter);
            }
        }

        private static void DemoArrayList()
        {
            ArrayList customers = new ArrayList();

            customers.Add(new Customer());
            //customers.Add(new Employee());
            //customers.Add(15.5f);

            foreach (Customer cust in customers)
            {
                Console.WriteLine(cust.ToString());
            }
        }

        private static void DemoListOfCustomer()
        {
            List<Customer> customers = new List<Customer>();

            customers.Add(new Customer());
            customers.Add(new Customer());
            //customers.Add(new Employee());

            foreach (Customer cust in customers)
            {
                Console.WriteLine(cust.ToString());
            }
        }

        private static void DemoArrayListBoxingPenalty()
        {
            ArrayList myInts = new ArrayList();

            myInts.Add(1);
            myInts.Add(2);
            myInts.Add(3);

            foreach (int myInt in myInts)
            {
                Console.WriteLine(myInt);
            }
        }

        private static void DemoListAvoidingBoxingPenalty()
        {
            List<int> myInts = new List<int>();

            myInts.Add(1);
            myInts.Add(2);
            myInts.Add(3);

            foreach (int myInt in myInts)
            {
                Console.WriteLine(myInt);
            }
        }

        private static void DemoShapeList()
        {
            List<Shape> myShapes = new List<Shape>();

            // because Box and Circle are type Shape
            //	they can be added to the collection too
            myShapes.Add(new Shape());
            myShapes.Add(new Box());
            myShapes.Add(new Circle());

            // Name is read polymorphically
            foreach (Shape shape in myShapes)
            {
                Console.WriteLine(shape.Name);
            }
        }

        static void DemoGenericSingleLinkedList()
        {
            // creates an instance of my custom generic type.
            // it is a collection of strings.
            GenericSingleLinkedList<string> singleLL = new GenericSingleLinkedList<string>();

            //
            // exercise the collection to make sure it works
            //

            singleLL.Insert(0, "zero");
            singleLL.Insert(1, "one");
            singleLL.Insert(2, "two");
            singleLL.Insert(3, "three");

            singleLL.Insert(2, "inserted");

            singleLL.RemoveAt(0);
            singleLL.RemoveAt(3);

            singleLL.Remove("inserted");

            // after modification use other methods
            // to examine the state of the collection
            Console.WriteLine("Contains two?:   {0}", singleLL.Contains("two"));
            Console.WriteLine("Contains three?: {0}", singleLL.Contains("three"));
            Console.WriteLine("IndexOf two:     {0}", singleLL.IndexOf("two"));

            // copy the list
            string[] myStrings = new string[singleLL.Count];
            singleLL.CopyTo(myStrings, 0);

            Console.WriteLine("Printing contents of array from CopyTo.");

            // iterate through the list to verify the copy worked
            foreach (string name in myStrings)
            {
                Console.WriteLine(name);
            }

            // use the Add method to make sure it works too
            singleLL.Add("four");

            // check out the indexer
            Console.WriteLine("Value at singleLL[1]: {0}", singleLL[1]);

            // make sure the collection holds the values we expect
            Console.WriteLine("Printing contents of singleLL.");
            foreach (string name in singleLL)
            {
                Console.WriteLine(name);
            }

            // verify that the Clear method works
            singleLL.Clear();
            Console.WriteLine("Size of singleLL: {0}", singleLL.Count);
        }

        private static void DemoGenericInterface()
        {
            Person cust1 = new Person();
            Person cust2 = new Person();

            cust1.Name = "Joe";
            cust2.Name = "Joe";

            Console.WriteLine(cust1.Equals(cust2));
        }

        private static void DemoGenericDelegate()
        {
            Clock clock = new Clock();
            Alarm alarm = new Alarm();

            clock.Tick += alarm.ListenToTick;

            clock.RaiseTick();
        }

        private static void DemoConstraints()
        {
            string headerFormat = @"
***************************************
*                                     
* {0}
*
***************************************
";

            ConstraintDemo constraints = new ConstraintDemo();

            Console.WriteLine(headerFormat, "Test Interface Constraints");

            // show how to work with interface constraints
            constraints.TestInterfaceConstraints();

            Console.WriteLine(headerFormat, "Test Base Class Constraints");

            // show how to work with base class constraints
            constraints.TestBaseClassConstraints();

            Console.WriteLine(headerFormat, "Test Reference and Value Type Constraints");

            // show how to work with reference
            // and value type constraints
            constraints.TestReferenceAndValueTypeConstraints();

            Console.WriteLine(headerFormat, "Test Constructor Constraints");

            // show how to work with constructor constraints
            constraints.TestConstructorConstraints();

            Console.WriteLine(headerFormat, "Test Constraints on Classes");

            //
            // run the Generic Type Constrains Demo
            //

            CustomerProcessor<Customer, Order> custProc =
                new CustomerProcessor<Customer, Order>();

            LargBiz largeCust = new LargBiz();
            MailOrder mailOrd = new MailOrder();

            custProc.ProcessOrder(largeCust, mailOrd);
        }
    }
}
