﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Chapter_19
{
    class Program
    {
        static void Main(string[] args)
        {
            List<StaffMember> hospitalStaff = new List<StaffMember>
            {
                new StaffMember { Name = "Marcus", Position="Doctor" },
                new StaffMember { Name = "Nancy", Position = "Nurse" },
                new StaffMember { Name = "Chris", Position = "Nurse" }
            };

            // basic query

            IEnumerable<StaffMember> staffMbrList =
                from staffMbr in hospitalStaff
                select staffMbr;

            foreach (var mbr in staffMbrList)
            {
                Console.WriteLine(mbr.Name);
            }

            // projection

            IEnumerable<StaffMbrAbbr> staffMbrAbbrList =
                from staffMbr in hospitalStaff
                select new StaffMbrAbbr { Name = staffMbr.Name };

            var staffNameList =
                from staffMbr in hospitalStaff
                select new { Name = staffMbr.Name };

            foreach (var mbr in staffNameList)
            {
                Console.WriteLine(mbr.Name);   
            }

            // filtering

            var nurses =
                from staffMbr in hospitalStaff
                where staffMbr.Position == "Nurse"
                select staffMbr.Name;

            foreach (var name in nurses)
            {
                Console.WriteLine(name);
            }

            // ordering

            var orderedNurses =
                from staffMbr in hospitalStaff
                where staffMbr.Position == "Nurse"
                orderby staffMbr.Name
                select staffMbr.Name;

            foreach (var nurse in orderedNurses)
            {
                Console.WriteLine("Name: {0}", nurse);
            }

            var orderedStaff =
                from staffMbr in hospitalStaff
                orderby staffMbr.Position descending, staffMbr.Name
                select new { staffMbr.Name, staffMbr.Position };

            foreach (var staff in orderedStaff)
            {
                Console.WriteLine("Name: {0}, Position: {1}", staff.Name, staff.Position);
            }

            // grouping

            var groupedStaff =
                from staffMbr in hospitalStaff
                group staffMbr by staffMbr.Position into staffMbrGroup
                select staffMbrGroup.Key;

            foreach (var staff in groupedStaff)
            {
                Console.WriteLine("Group: {0}", staff);
            }

            // joining

            List<PatientObject> patients = new List<PatientObject>
            {
                new PatientObject { Doctor = "Marcus", Name = "George" },
                new PatientObject { Doctor = "Doolittle", Name = "Cat" },
                new PatientObject { Doctor = "Marcus", Name = "Jane" }
            };

            var patientsAndDoctors =
                from doctor in hospitalStaff
                join patient in patients on doctor.Name equals patient.Doctor
                select new { Doctor = doctor.Name, Patient = patient.Name };

            foreach (var patientDoc in patientsAndDoctors)
            {
                Console.WriteLine("Doctor: {0}, Patient: {1}",
                        patientDoc.Doctor, patientDoc.Patient); 
            }

            var patientsGroupedByDoctors = 
                from doctor in hospitalStaff
                join patient in patients on doctor.Name equals patient.Doctor into PatientGroup
                select new { Doctor = doctor.Name, Patients = PatientGroup };

            foreach (var doc in patientsGroupedByDoctors)
            {
                foreach (var patient in doc.Patients)
                {
                    Console.WriteLine("Patient: {0}", patient.Name);
                }
            }

            // LINQ to SQL

            var hospitalCtx = new HospitalDataContext();

            var docs =
                from doctor in hospitalCtx.HospitalStaffs
                where doctor.Position == "Doctor"
                select doctor;

            foreach (var doc in docs)
            {
                Console.WriteLine("Doctor: {0}", doc.Name);
            }

            // Adding

            //hospitalCtx.HospitalStaffs.InsertOnSubmit(
            //    new HospitalStaff { Name = "John", Position = "Doctor" });

            // Updating

            //IQueryable<HospitalStaff> doctorsToUpdate =
            //    from doctor in hospitalCtx.HospitalStaffs
            //    where doctor.Name == "Marcus"
            //    select doctor;

            //doctorsToUpdate.First().Name = "Marcus MD";
            ////doctorsToUpdate.First().Position = "Nurse";

            // deleting

            //IQueryable<HospitalStaff> doctorsToDelete =
            //    from doctor in hospitalCtx.HospitalStaffs
            //    where doctor.Name == "Phil"
            //    select doctor;

            //hospitalCtx.HospitalStaffs.DeleteOnSubmit(doctorsToDelete.First());

            // submitting

            hospitalCtx.SubmitChanges();

            var docModified =
                from doctor in hospitalCtx.HospitalStaffs
                where doctor.Position == "Doctor"
                select doctor;

            foreach (var doc in docModified)
            {
                Console.WriteLine("Member: {0}", doc.Name);
            }

            // stored procs

            var doctors = hospitalCtx.GetDoctors();
            
            foreach (var doc in doctors)
            {
                Console.WriteLine("Doctor: {0}", doc.Name);
            }

            // functions

            var docNames = 
                from docName in hospitalCtx.Doctors()
                select docName.Name;

            foreach (var name in docNames)
            {
                Console.WriteLine("Doctor Name: {0}", name);
            }

            // modifications with stored procs


            // standard query operators

            // sort operators

            var staffRev =
                (from staff in hospitalStaff
                 select staff).Reverse();

            foreach (var staffMbr in staffRev)
            {
                Console.WriteLine(staffMbr.Name);
            }

            // set operators

            hospitalStaff.Add(
                new StaffMember
                {
                    Name = "Chris",
                    Position= "Nurse"
                }
            );

            hospitalStaff.ForEach(
                staff => Console.WriteLine(
                    "Name: {0}, Position: {1}",
                    staff.Name, staff.Position));

            var distinctStaff =
                (
                    from staff in hospitalStaff
                    select staff
                )
                .Distinct(new StaffMember());

            foreach (var staff in distinctStaff)
            {
               Console.WriteLine(
                    "Name: {0}, Position: {1}",
                    staff.Name, staff.Position);
            }

            hospitalStaff.Add(
                new TempStaffMember
                {
                    Name = "Pat",
                    Position = "Visiting Intern"
                }
            );

            var ofTypeStaff =
                (
                    from staff in hospitalStaff
                    select staff
                )
                .OfType<TempStaffMember>();

            foreach (var ofTypeMbr in ofTypeStaff)
            {
                Console.WriteLine("Name: {0}", ofTypeMbr.Name);
            }

            bool anyVeterinarians =
                (
                    from staff in hospitalStaff
                    select staff
                )
                .Any(staff => staff.Position == "Veterinarian");

            Console.WriteLine("Any Veterinarians: {0}", anyVeterinarians);

            var manyPatients =
                hospitalCtx.HospitalStaffs.SelectMany(
                    staff => staff.Patients);

            foreach (var patient in patients)
            {
                Console.WriteLine("Patient Name: {0}", patient.Name);
            }

            int startRecord = 0;
            int recordsPerPage = 3;

            var page = hospitalStaff
                .Skip(startRecord)
                .Take(recordsPerPage);

            foreach (var staff in page)
            {
                Console.WriteLine("Staff Member: {0}", staff.Name);
            }

            var groupJoinStaff =
                hospitalCtx.HospitalStaffs.GroupJoin(
                    hospitalCtx.Patients,
                    staff => staff,
                    patient => patient.HospitalStaff,
                    (staff, patientList) =>
                         new
                         {
                             StaffMemberName = staff.Name,
                             Patients = patientList
                         });

            foreach (var staff in groupJoinStaff)
            {
                Console.WriteLine("Doctor: {0}: ", staff.StaffMemberName);
                foreach (var patient in staff.Patients)
                {
                    Console.WriteLine("    Patient: {0}", patient.Name);
                }
            }

            var lookup =
                hospitalStaff.ToLookup(staff => staff.Position);

            foreach (var lkUp in lookup)
            {
                Console.WriteLine("Key: {0}", lkUp.Key);

                foreach (var staff in lkUp)
                {
                    Console.WriteLine("    Name: {0}", staff.Name);
                    
                }
            }

            var repeatedPatients =
                Enumerable.Repeat<PatientObject>(
                    new PatientObject
                    {
                        Name = "John",
                        Doctor = "Marcus"
                    },
                    20
                );

            foreach (var patient in repeatedPatients)
            {
                Console.WriteLine("Repeated Patient Name: {0}", patient.Name);
            }

            var staffMemberCopy = new List<StaffMember>
            {
                new StaffMember { Name = "Marcus", Position="Doctor" },
                new StaffMember { Name = "Nancy", Position = "Nurse" },
                new StaffMember { Name = "Chris", Position = "Nurse" }
            };

            bool areEqual = 
                hospitalStaff.SequenceEqual(staffMemberCopy);

            Console.WriteLine("SequenceEqual: {0}", areEqual);

            var doctorHyde =
                hospitalStaff.SingleOrDefault(
                    staff => 
                        staff.Name == "Hyde" && 
                        staff.Position == "Doctor");

            hospitalStaff.ToList().ForEach(
                staff => Console.WriteLine(
                    "List Name: {0}", staff.Name));

            var concatStaff = hospitalStaff.Concat(staffMemberCopy);

            foreach (var staff in concatStaff)
            {
                Console.WriteLine("Concat Name: {0}", staff.Name);
            }

            int numberOfStaff =
                (
                    from doc in hospitalStaff
                    where doc.Position == "Doctor"
                    select doc
                )
                .Count();

            Console.ReadKey();
        }
    }
}
