using System;
using System.Xml;
using System.Text;
using System.Collections;
using System.Collections.Generic;

/// <summary>
///    manages a list of money related
///    phrases to be obtained by a client
/// </summary>
public class MoneyTalk
{
    public const string TalkRoot = "moneyTalk";
    public const string TalkNode = "talk";

    public string FileName { get; set; }

    private List<string> m_talkList = new List<string>();

    /// <summary>
    ///		write XML data to a file
    /// </summary>
    public void WriteXML()
    {
        using (var xr = new XmlTextWriter(FileName, null))
        {
            xr.Formatting = Formatting.Indented;
            xr.Indentation = 4;

            xr.WriteStartDocument();
            xr.WriteComment("Holds data for the MoneyTalk program.");

            xr.WriteStartElement("moneyTalk");

            xr.WriteElementString(TalkNode, "A penny saved is too small, make it a buck.");
            xr.WriteElementString(TalkNode, "Keep your wooden nickel.  It'll be worth something someday.");
            xr.WriteElementString(TalkNode, "It's your dime, but you're better off dialing 10-10-XXX.");

            xr.WriteEndElement();

            xr.Flush();
        }
    }

    /// <summary>
    ///		read XML data from a file
    /// </summary>
    public void ReadXML()
    {
        using (var xr = new XmlTextReader(FileName))
        {
            string nodeName;

            while (xr.Read())
            {
                nodeName = xr.Name;

                if (nodeName == TalkNode)
                {
                    m_talkList.Add(xr.ReadString());
                }
            }
        }
    }

    /// <summary>
    ///		get a phrase
    /// </summary>
    public string Talk
    {
        get
        {
            return m_talkList[
                new Random().Next(m_talkList.Count)];
        }
    }
}
