﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Data.WebClient;
using HospitalModel;

namespace Chapter_23
{
    class Program
    {
        static void Main(string[] args)
        {
            QueryDataService();
            InsertDataServiceEntity();
            UpdateDataServiceEntity();
            DeleteDataServiceEntity();
            PerformLinqQueryWithWebDataContext();
            PerformLinqQueryWithHospitalEntities();

            Console.ReadKey();
        }

        private static void PerformLinqQueryWithHospitalEntities()
        {
            var hospital = new HospitalEntities(
                new Uri("http://localhost:1778/HospitalDataService.svc"));

            var docs =
                from doc in hospital.HospitalStaff
                where doc.Position == "Doctor"
                select new { doc.Name };

            docs.ToList().ForEach(
                doc => Console.WriteLine(
                    "LINQ with HospitalEntities - Name: {0}", doc.Name));
        }

        private static void PerformLinqQueryWithWebDataContext()
        {
            var hospital = new WebDataContext(
                new Uri("http://localhost:1778/HospitalDataService.svc"));

            var docs =
                from doc in hospital.CreateQuery<HospitalStaff>("/HospitalStaff")
                where doc.Position == "Doctor"
                select new { doc.Name };

            docs.ToList().ForEach(
                doc => Console.WriteLine(
                    "LINQ with WebDataContext - Name: {0}", doc.Name));
        }

        private static void DeleteDataServiceEntity()
        {
            var webCtx = new WebDataContext(
                new Uri("http://localhost:1778/HospitalDataService.svc"));

            webCtx.MergeOption = MergeOption.AppendOnly;

            var docs = webCtx.CreateQuery<HospitalStaff>(
                "/HospitalStaff?$filter=Name eq 'No'");

            docs.ToList().ForEach(
                doc => webCtx.DeleteObject(doc));

            webCtx.SaveChanges();

            docs = webCtx.CreateQuery<HospitalStaff>(
                "/HospitalStaff");

            docs.ToList().ForEach(
                doc => Console.WriteLine(
                    "Delete - Name: {0}, Position: {1}",
                    doc.Name, doc.Position));
        }

        private static void UpdateDataServiceEntity()
        {
            var webCtx = new WebDataContext(
                new Uri("http://localhost:1778/HospitalDataService.svc"));

            webCtx.MergeOption = MergeOption.AppendOnly;

            var docs = webCtx.CreateQuery<HospitalStaff>(
                "/HospitalStaff?$filter=Name eq 'No'&$top=1");

            var drNo = docs.ToArray()[0];
            drNo.Position = "Nurse";
            webCtx.UpdateObject(drNo);

            webCtx.SaveChanges();

            docs = webCtx.CreateQuery<HospitalStaff>(
                "/HospitalStaff");

            docs.ToList().ForEach(
                doc => Console.WriteLine(
                    "Update - Name: {0}, Position: {1}", 
                    doc.Name, doc.Position));
        }

        private static void InsertDataServiceEntity()
        {
            var webCtx = new WebDataContext(
                new Uri("http://localhost:1778/HospitalDataService.svc"));

            webCtx.AddObject("HospitalStaff",
                new HospitalStaff
                {
                    Name = "No",
                    Position = "Doctor"
                });

            webCtx.SaveChanges();

            var docs = webCtx.CreateQuery<HospitalStaff>(
                "/HospitalStaff?$filter=Position eq 'Doctor'");

            docs.ToList().ForEach(
                doc => Console.WriteLine(
                    "Insert - Name: {0}, Position: {0}", 
                    doc.Name, doc.Position));

        }

        /// <summary>
        /// uses LINQ to query a data service
        /// </summary>
        private static void QueryDataService()
        {
            var webCtx = new WebDataContext(
                new Uri("http://localhost:1778/HospitalDataService.svc"));

            var docs = webCtx.CreateQuery<HospitalStaff>("/HospitalStaff");

            docs.ToList().ForEach(
                doc => Console.WriteLine("Name: {0}", doc.Name));
        }
    }
}
